/*
 * Menu events :
 *   "Restore only the current session" on Tools Menu
 *   Create menuitems for Tab Bar Context Menu
 *   On Tab Bar Context Menu showing
 *   On Content Area Menu showing
 */
 
// Tools Menu : Restore only the current session
//
TabMix.toggleResumeSessionOnce = function () {
  var pref = "browser.sessionstore.resume_session_once";
  this.prefs.setBoolPref(pref, !getBoolPref(pref));
};

// Check the this menuitem when opening the Tool Menu
//
TabMix.checkResumeSessionOnceMenu = function () {
  var toolSR = document.getElementById("tm-tool-restoreSessionOnce");
  var appSR  = document.getElementById("tm-appmenu-restoreSessionOnce");
  var __PBS = Components.classes["@mozilla.org/privatebrowsing;1"]
                        .getService(Components.interfaces.nsIPrivateBrowsingService); 
  
  toolSR.hidden = !__TM_Prefs.getBoolPref("toolsSR") || __PBS.privateBrowsingEnabled;
  toolSR.setAttribute("checked", getBoolPref("browser.sessionstore.resume_session_once") );
  
  if ( appSR ) {
    appSR.hidden  = !__TM_Prefs.getBoolPref("toolsSR") || __PBS.privateBrowsingEnabled;
    appSR.setAttribute("checked", getBoolPref("browser.sessionstore.resume_session_once") );
  }
};

// On Content Area Menu showing
//
TabMix.onContentMenu = function () {
  var showUndoCloseTab  = __TM_Prefs.getBoolPref("undoCloseTabContent");
  var showUndoCloseList = __TM_Prefs.getBoolPref("undoCloseListContent");

  var closedTabCount    = TabMix._Ss.getClosedTabCount(window);
  var undoCloseTabMenu  = document.getElementById("tm-content-undoCloseTab");

  var undoCloseListMenu = document.getElementById("tm-content-undoCloseList");
  var undoCloseSep      = document.getElementById("tm-content-undoCloseSep");
  var contentClick      = gContextMenu.isTextSelected || gContextMenu.onTextInput ||
                          gContextMenu.onLink || gContextMenu.onImage;

  undoCloseTabMenu.hidden = !closedTabCount || !showUndoCloseTab || contentClick;
  undoCloseListMenu.hidden = !closedTabCount || !showUndoCloseList || contentClick;
  undoCloseSep.hidden = undoCloseTabMenu.hidden && undoCloseListMenu.hidden;
};

// On Tab Bar Context Menu showing
//
TabMix.onTabbarContext = function () {
  let closedTabCount    = TabMix._Ss.getClosedTabCount(window);
  let duplicateMenu     = document.getElementById("tm-duplicateTab");
  let protectMenu       = document.getElementById("tm-protectTab");
  let tabGroupMenu      = document.getElementById("tm-tabGroup");
  let undoCloseListMenu = document.getElementById("tm-undoCloseList");
  let closeRightMenu    = document.getElementById("tm-closeRightTabs");
  let closeLeftMenu     = document.getElementById("tm-closeLeftTabs");
  //let closeSepMenu      = document.getElementById("tm-separator");
  let pinTab            = document.getElementById("context_pinTab");
  let moveToGroupMenu   = document.getElementById("context_tabViewMenu");
  let moveToNewWindow   = document.getElementById("context_openTabInWindow");

  let tab     = TabContextMenu.contextTab;
  let nextTab = TabMix.nextTab(tab);
  let prevTab = TabMix.prevTab(tab);
  
  let firstOrLastTab = ( !nextTab || !prevTab );
  
  duplicateMenu.hidden     = !__TM_Prefs.getBoolPref("duplicateMenu");
  protectMenu.hidden       = !__TM_Prefs.getBoolPref("protectMenu");
  moveToGroupMenu.hidden   = !__TM_Prefs.getBoolPref("showMoveToGroup");
  moveToNewWindow.hidden   = !__TM_Prefs.getBoolPref("showMoveToNewWindow");
  closeRightMenu.hidden    = !__TM_Prefs.getBoolPref("closeRightMenu") || firstOrLastTab;
  closeLeftMenu.hidden     = !__TM_Prefs.getBoolPref("closeLeftMenu") || firstOrLastTab;
  undoCloseListMenu.hidden = !__TM_Prefs.getBoolPref("undoCloseListMenu") || !closedTabCount;
  //closeSepMenu.hidden      = closeRightMenu.hidden && closeLeftMenu.hidden;
  
  if ( tab.hasAttribute("hidden") )
    pinTab.disabled = true;
  
  var singleOrphan = !tab.hasAttribute("groupID") &&
                     ( !nextTab || nextTab.hasAttribute("groupID") ) &&
                     ( !prevTab || prevTab.hasAttribute("groupID") );
  tabGroupMenu.hidden = !__TM_Prefs.getBoolPref("tabGroup") || singleOrphan || tab.pinned;
  
  if ( tab.getAttribute("protected") == "true" )
    protectMenu.setAttribute("checked", "true");
  else
    protectMenu.removeAttribute("checked");
};

// On Tab Grouping Menu showing
//
TabMix.onTabGroupMenu = function () {
  var stackGroup      = document.getElementById("tm-groupFolder");
  var bookmarkGroup   = document.getElementById("tm-bookmarkGroup");
  var reloadGroup     = document.getElementById("tm-reloadGroup");
  var removeFromGroup = document.getElementById("tm-removeFromGroup");
  var groupSep        = document.getElementById("tm-context-groupSep");
  var undoGroup       = document.getElementById("tm-undoGroup");
  var closeGroup      = document.getElementById("tm-closeGroup");
  var groupOrphans    = document.getElementById("tm-groupOrphans");
  
  var stacked = TabContextMenu.contextTab.hasAttribute("hidden");
  var grouped = TabContextMenu.contextTab.hasAttribute("groupID");
  groupOrphans.hidden    = grouped;
  
  stackGroup.hidden      = !grouped;
  bookmarkGroup.hidden   = !grouped;
  
  removeFromGroup.hidden = stacked || !grouped;
  reloadGroup.hidden     = stacked || !grouped;
  groupSep.hidden        = stacked || !grouped;
  undoGroup.hidden       = stacked || !grouped;
  closeGroup.hidden      = stacked || !grouped;
};

// Colorize the grouped tabs
//
TabMix.onAllTabsPopupShown = function (event) {
  if ( !event.isTrusted )
    return;

  for ( let i = 0; i < this.childNodes.length; i++ ) {
    if ( this.childNodes[i].tab ) {
      let tab = this.childNodes[i].tab;
      if ( tab.hasAttribute("groupID") )
        this.childNodes[i].setAttribute( "groupID", tab.getAttribute("groupID") );
    }
  }
};

/*
 * Closed Tabs List :
 *   Create the Closed Tabs List Menu
 *   Clear all closed tabs
 */
TabMix.createUndoCloseList = function (popup) {
  while(popup.hasChildNodes()) {
    popup.removeChild(popup.lastChild);
  }
  
  var undoItems = JSON.parse(TabMix._Ss.getClosedTabData(window));
  for (var i = 0; i < undoItems.length; i++) {
    var m = popup.appendChild(document.createElement("menuitem"));
    m.setAttribute("class", "menuitem-iconic bookmark-item");
    m.setAttribute("label", i + ": " + undoItems[i].title);
    m.setAttribute("accesskey", i.toString());
    m.setAttribute("image", undoItems[i].image);
    m.setAttribute("oncommand", "undoCloseTab(" + i + ");");
  }
};

TabMix.clearAllClosedTabs = function () {
  var max = this.prefs.getIntPref("browser.sessionstore.max_tabs_undo");
  this.prefs.setIntPref("browser.sessionstore.max_tabs_undo",0);
  this.prefs.setIntPref("browser.sessionstore.max_tabs_undo",max);
};
 

