/*
 * Link event
 *   Handle the links with a |target| attribute
 */
TabMix._existsFrameName = function (containerFrame, targetFrame) {
  for (var i = 0; i < containerFrame.length; ++i) {
    if (containerFrame[i].name == targetFrame)
      return true;
    if (containerFrame[i].frames.length)
      var return_var = this._existsFrameName(containerFrame[i].frames,targetFrame);
  }
  if (return_var)
    return return_var;

  return false;
};

TabMix.onContentAreaClick = function (event) {
  var tmPref = __TM_Prefs.getIntPref("target");
  if ( tmPref == TabMix._TargetLink_Native || !(event.target instanceof HTMLElement) || 
       event.button != 0 || event.ctrlKey || event.shiftKey || event.altKey || event.metaKey )
    return;
  
  var [href, linkNode] = hrefAndLinkNodeForClickEvent(event);
  
  if ( !linkNode )
    return;

  let target = linkNode.target;
  if ( target && target != "_content" && target != "_main" &&
       target != "_self" && target != "_parent" && target != "_top" &&             
       !TabMix._existsFrameName(document.commandDispatcher.focusedWindow.top.frames, target) ) {
    if ( tmPref == TabMix._TargetLink_Current )
      linkNode.setAttribute("target", "_top");
    else
      linkNode.setAttribute("target", "_blank");
  }
};

/*
 * Mouse events :
 *   Click on Tab
 *   Ctrl-click on Tab and Tab Bar
 *   Double-click on Tab
 */
TabMix._tabClickAction = function ( aPref, aTab ) {
  switch (aPref) {
    case this._ClickTab_Null :
      break;
    case this._ClickTab_OpenNewTab :
      BrowserOpenTab();
      break;
    case this._ClickTab_RemoveTab :
      gBrowser.removeCurrentTab();
      break;
    case this._ClickTab_ProtectTab :
      this.protectTab(aTab);
      break;
    case this._ClickTab_PinTab :
      this.pinOrUnpinTab(aTab);
      break;
    case this._ClickTab_DuplicateTab :
      this.duplicateTab(aTab);
      break;
    case this._ClickTab_ReloadTab :
      BrowserReload();
      break;
    case this._ClickTab_TabGroup :
      this.toggleFolder(aTab);
      break
  }
};

TabMix.onTabBarDblClick = function (event) {
  if ( event.button != 0 || event.ctrlKey || event.metaKey || event.shiftKey || event.altKey )
    return;

  // Double-click on Tab Bar, then do nothing.
  // Firefox will open a new tab itself.
  //
  if ( event.target.localName != "tab" )
    return;
  
  var pref = __TM_Prefs.getIntPref("dblClickTab");
  TabMix._tabClickAction( pref, event.target );
};

TabMix.onClick = function (event) {
  var clickOutTabs = event.originalTarget.localName == "box" || event.originalTarget.localName == "scrollbox";
  var clickOnTab = event.target.localName == "tab";
  if ( !clickOutTabs )
    var tab = event.target;

  // Ctrl-click on Tab Bar will undo close tab
  if ( clickOutTabs && event.button == 0 && (event.ctrlKey || event.metaKey) ) {
    undoCloseTab();
    event.stopPropagation();
    event.preventDefault();
  }
  
  // Ctrl-click on a Tab
  if ( clickOnTab && event.button == 0 && (event.ctrlKey || event.metaKey) ) {
    var pref = __TM_Prefs.getIntPref("ctrlClickTab");
    TabMix._tabClickAction( pref, tab );
    event.stopPropagation();
  }

  // Alt-click on a Tab
  /* Disabled due to imcompatibility with Firefox button
  if ( clickOnTab && event.button == 0 && event.altKey ) {
    var pref = __TM_Prefs.getIntPref("altClickTab");
    TabMix._tabClickAction( pref, tab );
    event.stopPropagation();
  }*/

  // Shift-click on a Tab
  if ( clickOnTab && event.button == 0 && event.shiftKey ) {
    var pref = __TM_Prefs.getIntPref("shiftClickTab");
    TabMix._tabClickAction( pref, tab );
    event.stopPropagation();
  }

  // Expand a closed color group
  if ( clickOnTab && event.button == 0 && !event.ctrlKey && !event.metaKey && 
       event.originalTarget.className == "tab-content" &&
       tab.hasAttribute("groupID") && tab.hasAttribute("hidden") ) {
    TabMix.toggleFolder(tab);
    return;
  }

  // Click to select a Tab, see onMouseDown function below
  if ( clickOnTab && event.detail == 1 && event.button == 0 && !event.ctrlKey && !event.metaKey && 
       ( event.originalTarget.className != "tab-close-button" || tab.hasAttribute("protected") ) ) {
    gBrowser.selectedTab = tab;
  }

  // Long click on lock icon to unprotect Tab, see also onMouseDown
  if ( clickOnTab && event.detail == 1 && event.button == 0 && !event.ctrlKey && !event.metaKey && 
       event.originalTarget.className == "tab-close-button" && tab.hasAttribute("protected") ) {
    let time = Date.now();
    if ( tab.readyToUnprotect && time - tab.readyToUnprotect > TabMix.unprotectDelay )
      TabMix.protectTab(tab);
    tab.readyToUnprotect = null;
  }
};

TabMix.onMouseDown = function (event) {
  if ( __TM_Prefs.getBoolPref("clickFocus") && event.button == 0 &&
       event.target.localName == "tab" && event.originalTarget.className != "tab-close-button" )
    event.stopPropagation();

  if ( event.button == 0 && !event.ctrlKey && !event.metaKey && 
       event.target.localName == "tab" && event.originalTarget.className == "tab-close-button" &&
       event.target.hasAttribute("protected") ) {
    event.target.readyToUnprotect = Date.now();
  }
};

/*
 * Drag and drop to group orphan tabs
 */
TabMix.onDragOver = function (event) {
  if ( !TabMixGroup.enabled )
    return;

  let tab = event.target;
  let draggedTab = event.dataTransfer.mozGetDataAt(TAB_DROP_TYPE, 0);

  // a collapsed tab group
  if ( !draggedTab || 
       ( draggedTab.groupID && draggedTab.hasAttribute("hidden") ) ||
       ( draggedTab.groupID && tab.groupID == draggedTab.groupID ) )
    return;
  
  if ( tab.localName == "tab" && event.dataTransfer.dropEffect == "move" && tab != draggedTab ) {
    let boxObject = tab.boxObject;
    if ( event.screenX > boxObject.screenX + boxObject.width * .25 &&
         event.screenX < boxObject.screenX + boxObject.width * .75 ) {
      // this = tabContainer
      if ( !this._dragTime )
        this._dragTime = Date.now();
      if ( Date.now() >= this._dragTime + TabMix.dragOverDelay && !tab.hasAttribute("TMtoMakeGroup") ) {
        let color = TabMixGroup.pickColor(tab);
        tab.setAttribute( "TMtoMakeGroup", color );
        draggedTab.setAttribute( "TMtoMakeGroup", color );
      }
    }
    else {
      tab.removeAttribute("TMtoMakeGroup");
      draggedTab.removeAttribute("TMtoMakeGroup");
    }
  }
};

TabMix.onDragEnd = function (event) {
  if ( !TabMixGroup.enabled )
    return;

  Array.forEach( gBrowser.tabs, function (tab) {
    tab.removeAttribute("TMtoMakeGroup");
  });
};

TabMix.onDrop = function (event) {
  if ( !TabMixGroup.enabled )
    return;
  
  let dt = event.dataTransfer;

  if ( dt.dropEffect != "move" )
    return;
  
  let draggedTab = dt.mozGetDataAt(TAB_DROP_TYPE, 0);
  if (!draggedTab)
    return;
  
  let targetTab = event.target;
  
  TabMix._makeGroupOnDrop( draggedTab, targetTab );
};

// For new tab drag behaviour of Firefox 8+
TabMix.onTabDrag = function (event) {
  // this = tabContainer
  if ( !this.draggedTab || this.draggedTab.pinned || !TabMixGroup.enabled )
    return;
  
  // a collapsed tab group
  if ( this.draggedTab.groupID && this.draggedTab.hasAttribute("hidden") )
    return;
  
  let ltr = (window.getComputedStyle(this).direction == "ltr");
  
  let draggedTab = this.draggedTab;
  let tabWidth = draggedTab.getBoundingClientRect().width;
  let transformX = draggedTab._dragData._dragDistX;
  if ( !tabWidth )
    return;

  let move = transformX % tabWidth;
  let numShift = Math.abs( transformX - move ) / tabWidth + 1;
  if ( transformX < 0 ) {
    numShift *= -1;
    move *= -1;
  }
  if ( !ltr )
    numShift *= -1;
  
  this._dragTargetTab = this.tabbrowser.visibleTabs[draggedTab._tPos + numShift];
  let tab = this._dragTargetTab;

  if ( !tab || 
       tab.pinned ||
       ( draggedTab.groupID && tab.groupID == draggedTab.groupID ) )
    return TabMix._clearDragTab();;
    
  if ( move > tabWidth * .25 && move < tabWidth * .75 ) {
    if ( !this._dragTime )
      this._dragTime = Date.now();
    if ( Date.now() >= this._dragTime + TabMix.dragOverDelay && !tab.hasAttribute("TMtoMakeGroup") ) {
      let color = TabMixGroup.pickColor(tab);
      tab.setAttribute( "TMtoMakeGroup", color );
      draggedTab.setAttribute( "TMtoMakeGroup", color );
    }
  }
  else {
    TabMix._clearDragTab();
  }
};

TabMix.onTabDragScroll = function (event) {
  if ( !TabMixGroup.enabled )
    return;    
  TabMix._clearDragTab();
};

TabMix.onEndTabDrag = function (event) {
  if ( !TabMixGroup.enabled )
    return;
  TabMix._clearDragTab();    
};

TabMix.onTabDrop = function (event) {
  if ( !TabMixGroup.enabled )
    return;
  
  let tabContainer = gBrowser.tabContainer;
  if ( !tabContainer.draggedTab || !tabContainer._dragTargetTab )
    return;
  
  let draggedTab = tabContainer.draggedTab;
  let targetTab = tabContainer._dragTargetTab;
  
  TabMix._makeGroupOnDrop( draggedTab, targetTab );
};

TabMix._makeGroupOnDrop = function ( aDraggedTab, aTargetTab ) {
  if ( aTargetTab.hasAttribute("TMtoMakeGroup") ) {
    if ( aDraggedTab.groupID ) {
      if ( aDraggedTab.groupID == aTargetTab.groupID )
        return;
      TabMixGroup.removeOneTab(aDraggedTab);
    }

    aTargetTab.removeAttribute("TMtoMakeGroup");
    aDraggedTab.removeAttribute("TMtoMakeGroup");
    TabMixGroup.addFromTab( aDraggedTab, aTargetTab );
  }
};

TabMix._clearDragTab = function () {
  gBrowser.tabContainer._dragTime = 0;
  gBrowser.tabContainer._dragTargetTab = null;
  
  gBrowser.visibleTabs.forEach( function (tab) {
    tab.removeAttribute("TMtoMakeGroup");
  })
};

/*
 * Keyboard event :
 *   Alt-C will toggle the focus between the current tab and the page content
 */
TabMix.toggleFocus = function () {
  if ( !gBrowser.mStrip.collapsed && document.commandDispatcher.focusedElement != gBrowser.mCurrentTab )
    gBrowser.mCurrentTab.focus();
  else
    window.content.focus();
};

